Package: descr
Version: 1.1.9
Date: 2026-04-28
Title: Descriptive Statistics
Authors@R: c(
    person(given = "Jakson", family = "Aquino", role = "cre",
           email = "jalvesaq@gmail.com",
           comment = c(ORCID = "0000-0003-2522-8573")),
    person(given = "Dirk", family = "Enzmann", role = "aut"),
    person(given = "Marc", family = "Schwartz", role = "aut"),
    person(given = "Nitin", family = "Jain", role = "aut"),
    person(given = "Stefan", family = "Kraft", role = "aut"))
Imports: xtable, utils, grDevices, graphics, stats
Description: Weighted frequency and contingency tables of
    categorical variables and of the comparison of the mean
    value of a numerical variable by the levels of a factor,
    and methods to produce xtable objects of the tables and to
    plot them. There are also functions to facilitate
    character encoding conversion of objects, to quickly
    convert fixed-width files into CSV ones, and to export a
    data.frame to a text file with the necessary R and SPSS
    codes to reread the data.
License: GPL (>= 2)
URL: https://github.com/jalvesaq/descr
NeedsCompilation: yes
Packaged: 2026-04-28 11:33:35 UTC; aquino
Author: Jakson Aquino [cre] (ORCID: <https://orcid.org/0000-0003-2522-8573>),
  Dirk Enzmann [aut],
  Marc Schwartz [aut],
  Nitin Jain [aut],
  Stefan Kraft [aut]
Maintainer: Jakson Aquino <jalvesaq@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 12:10:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-01 13:00:21 UTC; unix
Archs: descr.so.dSYM
